@@ -1,5 +1,5 @@
 package POE::Component::Jabber::Client::Component;
-use POE::Preprocessor;
+use Filter::Template;
 const XNode POE::Filter::XML::Node
 use warnings;
 use strict;
@@ -11,7 +11,7 @@ use POE::Filter::XML::Node;
 use POE::Filter::XML::NS qw/ :JABBER :IQ /;
 use Digest::SHA1 qw/ sha1_hex /;
 
-our $VERSION = '1.1';
+our $VERSION = '1.21';
 
 sub new()
 {
@@ -73,6 +73,7 @@ sub new()
 
 		ServerInput => \&init_input_handler,
 		ServerError => \&server_error,
+		ConnectError => \&connect_error,
 
 		InlineStates => {
 			output_handler => \&output_handler,
@@ -88,6 +89,16 @@ sub new()
 	);
 }
 
+sub connect_error()
+{
+	my ($kernel, $heap, $call, $code, $err) = @_[KERNEL, HEAP, ARG0..ARG2];
+
+	warn "Connect Error: $call: $code -> $err\n";
+	$kernel->post($heap->{'CONFIG'}->{'state_parent'},
+		$heap->{'CONFIG'}->{'states'}->{'errorevent'},
+		+PCJ_CONNFAIL, $call, $code, $err);
+}
+
 sub reconnect_to_server()
 {
 	my ($kernel, $heap, $session, $ip, $port) =
@@ -271,7 +282,7 @@ sub server_error()
 
 sub debug_message()
 {
-	print STDERR "\n", scalar (localtime (time)), ": ". shift ."\n";
+	warn "\n", scalar (localtime (time)), ": ". shift(@_) ."\n";
 }
 
 1;
